<?php
/**
* @copyright  Copyright 2018 © Intelligent IT SRL. All rights reserved.
*/

class SmartBillSettings {

    public static $auth_fields = ['SMARTBILL_EMAIL', 'SMARTBILL_API_TOKEN', 'SMARTBILL_CIF'];
    public static $bool_fields = [ 'SMARTBILL_INVOICE_CASHING','SMARTBILL_AUTOMATICALLY_ISSUE_DOCUMENT','SMARTBILL_PUBLIC_INVOICE','SMARTBILL_SEND_MAIL_WITH_DOCUMENT','SMARTBILL_PRODUCT','SMARTBILL_ISSUE_WITH_DELIVERY_DAYS','SMARTBILL_SAVE_STOCK_HISTORY','SMARTBILL_SYNC_STOCK', 'SMARTBILL_USE_PAYMENT_TAX', 'SMARTBILL_PRICE_INCLUDE_DISCOUNTS', 'SMARTBILL_INCLUDE_SHIPPING', 'SMARTBILL_SAVE_PRODUCT', 'SMARTBILL_ADD_DELEGATE_DATA','SMARTBILL_SAVE_CLIENT','SMARTBILL_PAYMENT_URL','SMARTBILL_INVOICE_IS_DRAFT','SMARTBILL_ISSUE_WITH_DUE_DATE', 'SMARTBILL_DEBUG'];
    public static $select_fields = ['SMARTBILL_DOCUMENT_DATE', 'SMARTBILL_FREE_SHIPPING', 'SMARTBILL_ORDER_STATUS', 'SMARTBILL_PRICES_VAT', 'SMARTBILL_TRANSPORT_VAT','USE_INTRA_CIF', 'SMARTBILL_DOCUMENT_TYPE', 'SMARTBILL_INVOICE_SERIES', 'SMARTBILL_ESTIMATE_SERIES', 'SMARTBILL_PRODUCT_SKU_TYPE', 'SMARTBILL_UM', 'SMARTBILL_BILLING_CURRENCY','USE_INTRA_CIF','SMARTBILL_INVOICE_LANG', 'SMARTBILL_STOCK','SMARTBILL_USED_STOCK'];
    public static $input_fields = ['SMARTBILL_COUPON_TEXT', 'SMARTBILL_SEND_MAIL_CC','SMARTBILL_SEND_MAIL_BCC','SMARTBILL_DUE_DAYS','SMARTBILL_DELIVERY_DAYS','SMARTBILL_ISSUER_NAME','SMARTBILL_ISSUER_CNP','SMARTBILL_DELEGATE_NAME','SMARTBILL_DELEGATE_BULLETIN','SMARTBILL_DELEGATE_AUTO','SMARTBILL_SHIPPING_NAME'];
    public static $textarea_fields = ['SMARTBILL_SHOW_ORDER_MENTION', 'SMARTBILL_SHOW_ORDER_OBS'];

    public static $document_types = [[
        'value' => '0',
        'label' => 'Factura'
    ],
    [
        'value' => '1',
        'label' => 'Proforma']
    ];

    const SKU_REFERENCE         = 'reference';
    const SKU_EAN13_JAN         = 'ean13';
    const SKU_UPC               = 'upc';
    const SKU_ID                = 'product_id';

    /**
     * Function to return all settings field names as array
     *
     * @return array
     */
    public static function get_all_fields() {
        return array_merge(self::$auth_fields, self::$bool_fields, self::$select_fields, self::$input_fields, self::$textarea_fields);
    }

    /**
     * Function to return auth settings values
     *
     * @return array @arr
     */
    public static function get_auth_fields_values() {
        $arr = [];
        $auth_fields = self::$auth_fields;
        foreach ($auth_fields as $field) {
            $arr[$field] = Configuration::get($field);
        }
        return $arr;
    }

    /**
     * Function to return all settings values for Prestashop
     *
     * @return array @arr
     */
    public static function get_raw_config_fields_values() {
        $arr = [];
        $all_fields = self::get_all_fields();
        foreach ($all_fields as $field) {
            $arr[$field] = Configuration::get($field);
        }

        return $arr;
    }

    /**
     * Function to return all settings values for API
     *
     * @return array @arr
     */
    public static function get_config_fields_values() {
        $arr = [];
        $all_fields = self::get_all_fields();
        $bool_fields = self::$bool_fields;

        if(!Configuration::get('SMARTBILL_VAT_RATES')){
            $vat_rates =  self::get_vat_rates(Configuration::get('SMARTBILL_CIF'));
            Configuration::updateValue('SMARTBILL_VAT_RATES',serialize($vat_rates));
        }else{
            $vat_rates=unserialize(Configuration::get('SMARTBILL_VAT_RATES'));
        }


        foreach ($all_fields as $field) {
            if ($field == 'SMARTBILL_PRICES_VAT' || $field == 'SMARTBILL_TRANSPORT_VAT') {
                if (! is_array($vat_rates) || '' === Configuration::get($field) ) {
                    $arr[$field] = '';
                } else {
                    if(is_array($vat_rates) && ''!==Configuration::get($field) && isset($vat_rates[Configuration::get($field)])){
                        $vat_rates_selected = $vat_rates[Configuration::get($field)];
                        $arr[$field] = floatval($vat_rates_selected['percentage']);
                        $vatParse=str_replace('-','',$vat_rates_selected['name']);
                        $vatParse=trim($vatParse);
                        if($field == 'SMARTBILL_TRANSPORT_VAT' && $vatParse == 'Preluata din Prestashop'){
                            $arr[$field]="presta";
                        }
                        if($field == 'SMARTBILL_PRICES_VAT' && $vatParse == 'Preluata din Prestashop'){
                            $arr[$field]="Preluata din Prestashop";
                        }
                    }
                }
            } elseif ($field == 'SMARTBILL_UM' && empty(Configuration::get($field))) {
                $arr[$field] = 'no_value'; // Unit measure should never be empty
            } elseif ($field == 'SMARTBILL_BILLING_CURRENCY') {
                $arr[$field] = self::get_currencies(Configuration::get($field)); // Get actual currency value
            } elseif ($field == 'SMARTBILL_STOCK' && empty(Configuration::get($field))) {
                $arr[$field] = false;
            } elseif (in_array($field, $bool_fields)) {
                $arr[$field] = (bool)Configuration::get($field);
            } else {
                $arr[$field] = Configuration::get($field);
            }
        }

        return $arr;
    }

    /**
    * Function used to determine if company is VAT payable or not
    *
    * @return boolean $status_vat_payable
    */
    public static function is_vat_payable($vat_code = null) {

        $vat_rates = self::get_vat_rates($vat_code);
        if (! is_array($vat_rates)){
            $status_vat_payable = false;
        }
        else {
            $status_vat_payable = true;
        }
        return $status_vat_payable;
    }

    /**
    * Function used to get the VAT rates for a company
    *
    * @return string|array $vat_rates
    */
    public static function get_vat_rates($vat_code = null) {

        $auth_options = self::get_auth_fields_values();
        if (! is_null($vat_code)) {
            $auth_options['SMARTBILL_CIF'] = $vat_code;
        }

        try {
            if ( empty($auth_options['SMARTBILL_CIF']) ) {
                return false;
            }

            if(!empty(Configuration::get('SMARTBILL_NO_TAX'))){
                return Configuration::get('SMARTBILL_NO_TAX');
            }

            $client = new SmartBill_Cloud_REST_Client($auth_options['SMARTBILL_EMAIL'], $auth_options['SMARTBILL_API_TOKEN']);
            if(!Configuration::get('SMARTBILL_TAXES')){
                $taxes = $client->getTaxes($auth_options['SMARTBILL_CIF']);
                Configuration::updateValue('SMARTBILL_TAXES',serialize($taxes));
            }else{
                $taxes=unserialize(Configuration::get('SMARTBILL_TAXES'));
            }
            // get PS Tax rate
            $prod = new Product;
            $ps_tax_rate = $prod->getTaxesRate();

            if (is_array($taxes) && isset($taxes['taxes'])) {
                $taxes['taxes'][] = array(
                    'name' => 'Preluata din Prestashop',
                );
                return $taxes['taxes'];
            } else {
                return Context::getContext()->getTranslator()->trans('Firma este neplatitoare de TVA sau nu au fost setate valori de TVA in SmartBill Cloud', array(), 'Modules.Smartbill.Admin');
            }

        } catch (Exception $e) {
            return $e->getMessage();
        }

    }

    /**
    * Function used to get the measuring units for the company
    *
    * @return string|array $measuring_units
    */
    public static function get_measuring_units($vat_code = null) {

        $auth_options = self::get_auth_fields_values();
        if (! is_null($vat_code)) {
            $auth_options['SMARTBILL_CIF'] = $vat_code;
        }

        try {
            if ( empty($auth_options['SMARTBILL_CIF']) ) {
                return false;
            }

            $client = new SmartBill_Cloud_REST_Client($auth_options['SMARTBILL_EMAIL'], $auth_options['SMARTBILL_API_TOKEN']);
            $mu = $client->getMeasuringUnits($auth_options['SMARTBILL_CIF']);

            if ( is_array($mu) && isset($mu['mu'])){
                return $mu['mu'];
            } else {
                return Context::getContext()->getTranslator()->trans('Firma este neplatitoare de TVA sau nu au fost setate valori de TVA in SmartBill Cloud', array(), 'Modules.Smartbill.Admin');
            }

        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    /**
    * Function used to get the raw currencies
    *
    * @return array $currencies
    */
    public static function get_currencies($index = null) {
        $ps_currency = Currency::getDefaultCurrency();
        $ps_currency_value = $ps_currency->iso_code;
        $ps_currency_name = $ps_currency->name;
        $translator = Context::getContext()->getTranslator();
        $currencies = [
            ['currency' => "RON", 'label' => $translator->trans('RON - Leu', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "EUR", 'label' => $translator->trans('EUR - Euro', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "USD", 'label' => $translator->trans('USD - Dolar', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "GBP", 'label' => $translator->trans('GBP - Lira sterlina', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "CAD", 'label' => $translator->trans('CAD - Dolar canadian', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "AUD", 'label' => $translator->trans('AUD - Dolar australian', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "CHF", 'label' => $translator->trans('CHF - Franc elvetian', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "TRY", 'label' => $translator->trans('TRY - Lira turceasca', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "CZK", 'label' => $translator->trans('CZK - Coroana ceheasca', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "DKK", 'label' => $translator->trans('DKK - Coroana daneza', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "HUF", 'label' => $translator->trans('HUF - Forintul maghiar', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "MDL", 'label' => $translator->trans('MDL - Leu moldovenesc', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "SEK", 'label' => $translator->trans('SEK - Coroana suedeza', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "BGN", 'label' => $translator->trans('BGN - Leva bulgareasca', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "NOK", 'label' => $translator->trans('NOK - Coroana norvegiana', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "JPY", 'label' => $translator->trans('JPY - Yenul japonez', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "EGP", 'label' => $translator->trans('EGP - Lira egipteana', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "PLN", 'label' => $translator->trans('PLN - Zlotul polonez', array(), 'Modules.Smartbill.Admin')],
            ['currency' => "RUB", 'label' => $translator->trans('RUB - Rubla', array(), 'Modules.Smartbill.Admin')],
            ['currency' => $ps_currency_value, 'label' => $ps_currency_name . $translator->trans(' - Preluat din PrestaShop', array(), 'Modules.Smartbill.Admin')]
        ];

        if ( ! is_null ($index) && $index != '' ) {
            return $currencies[$index]['currency'];
        } else {
            $return = [];
            foreach ($currencies as $k => $currency) {
                $return[] = [
                    'value' => $k,
                    'label' => $currency['label'],
                ];
            }
            return $return;
        }
    }


    /**
    * Function used to get the raw languages
    *
    * @return array $languages
    */
    public static function get_languages() {
        $translator = Context::getContext()->getTranslator();
        $languages=[
            ['value'=>"RO","label"=>$translator->trans('Romana', array(), 'Modules.Smartbill.Admin')],
            ['value'=>"EN","label"=>$translator->trans('Engleza', array(), 'Modules.Smartbill.Admin')],
            ['value'=>"FR","label"=>$translator->trans('Franceza', array(), 'Modules.Smartbill.Admin')],
            ['value'=>"IT","label"=>$translator->trans('Italiana', array(), 'Modules.Smartbill.Admin')],
            ['value'=>"SP","label"=>$translator->trans('Spaniola', array(), 'Modules.Smartbill.Admin')],
            ['value'=>"HU","label"=>$translator->trans('Maghiara', array(), 'Modules.Smartbill.Admin')],
            ['value'=>"DE","label"=>$translator->trans('Germana', array(), 'Modules.Smartbill.Admin')]
        ];

        return $languages;
    }

    /**
    * Function used to get the used cif options
    *
    * @return array $cifs
    */
    public static function get_cifs() {
        $translator = Context::getContext()->getTranslator();
        $auth_options = self::get_auth_fields_values();
        $cifs=[
            ['value'=>"authcif", "label" => $auth_options['SMARTBILL_CIF']],
            ['value'=>"intracom", "label"=> $translator->trans('CIF intracomunitar', array(), 'Modules.Smartbill.Admin')]
        ];

        return $cifs;
    }


    /**
     * Function used to get the sku types to be used
     *
     * @return string|array $sku_types
     */
    public static function get_sku_types() {
        return array(
            array(
                'value'    => self::SKU_REFERENCE,
                'label'  => 'Reference Code',
            ),
            array(
                'value'    => self::SKU_EAN13_JAN,
                'label'  => 'EAN13 or JAN Barcode',
            ),
            array(
                'value'    => self::SKU_UPC,
                'label'  => 'UPC barcode',
            ),
            array(
                'value'    => self::SKU_ID,
                'label'  => 'ID',
            ),
        );
    }

    public static function get_warehouses($vat_code = null){
        $auth_options = self::get_auth_fields_values();
        if (! is_null($vat_code)) {
            $auth_options['SMARTBILL_CIF'] = $vat_code;
        }
        try {
            if ( empty($auth_options['SMARTBILL_CIF']) ) {
                return false;
            }

            $client = new SmartBill_Cloud_REST_Client($auth_options['SMARTBILL_EMAIL'], $auth_options['SMARTBILL_API_TOKEN']);

            $warehouses = $client->warehouses($auth_options['SMARTBILL_CIF']);
            $finalValues = [];
            if (is_array($warehouses)) {  
                foreach($warehouses as $warehouse){
                    $finalValues[$warehouse] = $warehouse;
                }
                return $finalValues;
            }
            else {
                throw new Exception(Context::getContext()->getTranslator()->trans("Raspuns invalid primit de la SmartBill Cloud la primirea informatiilor despre gestiuni.", array(), 'Modules.Smartbill.Admin'));
            }
        } catch (Exception $e) {
            return $e->getMessage();
        }
    }


    /**
     * Function used to get the stocks for the company
     *
     * @return string|array $stocks
     */
    public static function get_stock($vat_code = null) {

        $auth_options = self::get_auth_fields_values();
        if (! is_null($vat_code)) {
            $auth_options['SMARTBILL_CIF'] = $vat_code;
        }

        try {
            if ( empty($auth_options['SMARTBILL_CIF']) ) {
                return false;
            }

            $client = new SmartBill_Cloud_REST_Client($auth_options['SMARTBILL_EMAIL'], $auth_options['SMARTBILL_API_TOKEN']);
            $data = [
                "cif" => $auth_options['SMARTBILL_CIF'],
                "date" => date('Y-m-d'),
                "warehouseName" => "",
                "productName" => "",
                "productCode" => ""
            ];
            $stocks = $client->productsStock($data);
            $finalValues = [];
            if (is_array($stocks)) {
                Configuration::updateValue('smartbill_prod_stocks', serialize($stocks));
                foreach($stocks as $stock){
                    $item['value'] = $stock['warehouse']['warehouseName'];
                    $finalValues[$item['value']] = $item['value'];
                }
                return $finalValues;
            }
            else {
                throw new Exception(Context::getContext()->getTranslator()->trans("Raspuns invalid primit de la SmartBill Cloud la primirea informatiilor despre stocuri.", array(), 'Modules.Smartbill.Admin'));
            }
        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    /**
     * This function returns the invoice + estimate series from the WP DB and the SmartBill Cloud
     *
     * @return array
     */
    public static function get_series_settings() {
        $options = self::get_config_fields_values();

        try {
            if ( empty($options['SMARTBILL_CIF']) ) {
                return false;
            }

            if ( empty($options['SMARTBILL_EMAIL']) || empty($options['SMARTBILL_API_TOKEN'] ) ) {
                throw new \Exception(Context::getContext()->getTranslator()->trans('Este necesar sa furnizati un utilizator si o parola valide.', array(), 'Modules.Smartbill.Admin'));
            }

            $connector = new SmartBill_Cloud_REST_Client($options['SMARTBILL_EMAIL'], $options['SMARTBILL_API_TOKEN']);
            $raw_series = $connector->getDocumentSeries($options['SMARTBILL_CIF']);

            $document_series = [];
            if ( isset($raw_series['list']) && is_array($raw_series['list'])) {
                foreach ($raw_series['list'] as $v) {
                    $document_series[] = $v;
                }
            } else {
                throw new \Exception(Context::getContext()->getTranslator()->trans('Raspuns invalid primit de la SmartBill Cloud la primirea seriilor pentru facturi.', array(), 'Modules.Smartbill.Admin'));
            }

            return $document_series;

        } catch(Exception $e){
            $error = $e->getMessage();
            return $error;
        }
    }
}
